@@ -2,14 +2,14 @@ use crate::a2ml::*;
 use super::tokenizer::*;
 
 struct TokenIter<'a> {
-    tokens: &'a Vec<A2lToken>,
+    tokens: &'a [A2lToken],
     pos: usize
 }
 
 pub struct ParserState<'a> {
     token_cursor: TokenIter<'a>,
-    filedata: &'a Vec<String>,
-    filenames: &'a Vec<String>,
+    filedata: &'a [String],
+    filenames: &'a [String],
     last_token_position: u32,
     sequential_id: u32,
     pub(crate) log_msgs: &'a mut Vec<String>,
@@ -52,7 +52,7 @@ pub enum ParseError {
 
 // it pretends to be an Iter, but it really isn't
 impl<'a> TokenIter<'a> {
-    fn next(self: &mut Self) -> Option<&'a A2lToken> {
+    fn next(&mut self) -> Option<&'a A2lToken> {
         if self.pos < self.tokens.len() {
             let item = &self.tokens[self.pos];
             self.pos += 1;
@@ -62,7 +62,7 @@ impl<'a> TokenIter<'a> {
         }
     }
 
-    fn peek(self: &mut Self) -> Option<&'a A2lToken> {
+    fn peek(&mut self) -> Option<&'a A2lToken> {
         if self.pos < self.tokens.len() {
             let item = &self.tokens[self.pos];
             Some(item)
@@ -71,18 +71,18 @@ impl<'a> TokenIter<'a> {
         }
     }
 
-    fn back(self: &mut Self) {
+    fn back(&mut self) {
         self.pos -= 1;
     }
 }
 
 
 impl<'a> ParserState<'a> {
-    pub fn new<'b>(tokens: &'b Vec<A2lToken>, filedata: &'b Vec<String>, filenames: &'b Vec<String>, log_msgs: &'b mut Vec<String>, strict: bool) -> ParserState<'b> {
+    pub fn new<'b>(tokens: &'b [A2lToken], filedata: &'b [String], filenames: &'b [String], log_msgs: &'b mut Vec<String>, strict: bool) -> ParserState<'b> {
         ParserState {
             token_cursor: TokenIter{ tokens, pos: 0},
             filedata,
-            filenames: filenames,
+            filenames,
             last_token_position: 0,
             sequential_id: 0,
             log_msgs,
@@ -234,7 +234,7 @@ impl<'a> ParserState<'a> {
             let token = self.expect_token(context, A2lTokenType::String)?;
             let mut text = self.get_token_text(token);
 
-            if text.starts_with("\"") {
+            if text.starts_with('\"') {
                 text = &text[1..text.len() - 1];
             }
 
@@ -304,7 +304,7 @@ impl<'a> ParserState<'a> {
             match u8::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num as i8, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -321,7 +321,7 @@ impl<'a> ParserState<'a> {
             match u8::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -338,7 +338,7 @@ impl<'a> ParserState<'a> {
             match u16::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num as i16, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -355,7 +355,7 @@ impl<'a> ParserState<'a> {
             match u16::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -372,7 +372,7 @@ impl<'a> ParserState<'a> {
             match u32::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num as i32, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -389,7 +389,7 @@ impl<'a> ParserState<'a> {
             match u32::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -406,7 +406,7 @@ impl<'a> ParserState<'a> {
             match u64::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -423,7 +423,7 @@ impl<'a> ParserState<'a> {
             match u64::from_str_radix(&text[2..], 16) {
                 Ok(num) => Ok((num as i64, true)),
                 Err(_) => Err(ParseError::MalformedNumber(context.clone(), text.to_string()))
-            }        
+            }
         } else {
             match text.parse() {
                 Ok(num) => Ok((num, false)),
@@ -609,41 +609,125 @@ impl ParseContext {
 
 
 fn unescape_string(text: &str) -> String {
-    let input_chars: Vec<char> = text.chars().collect();
-    let mut output_chars = Vec::<char>::new();
+    /* first check if any unescaping is needed at all */
+    if text.chars().any(|c| c == '\\' || c == '"') {
+        let input_chars: Vec<char> = text.chars().collect();
+        let mut output_chars = Vec::<char>::new();
+
+        let mut idx = 1;
+        while idx < input_chars.len() {
+            if (input_chars[idx-1] == '\\' || input_chars[idx-1] == '"') && input_chars[idx] == '"' {
+                output_chars.push('"');
+                idx += 1;
+            } else if input_chars[idx-1] == '\\' && input_chars[idx] == '\'' {
+                output_chars.push('\'');
+                idx += 1;
+            } else if input_chars[idx-1] == '\\' && input_chars[idx] == '\\' {
+                output_chars.push('\\');
+                idx += 1;
+            } else if input_chars[idx-1] == '\\' && input_chars[idx] == 'n' {
+                output_chars.push('\n');
+                idx += 1;
+            } else if input_chars[idx-1] == '\\' && input_chars[idx] == 'r' {
+                output_chars.push('\r');
+                idx += 1;
+            } else if input_chars[idx-1] == '\\' && input_chars[idx] == 't' {
+                output_chars.push('\t');
+                idx += 1;
+            } else {
+                output_chars.push(input_chars[idx-1]);
+            }
 
-    let mut idx = 1;
-    while idx < input_chars.len() {
-        if input_chars[idx-1] == '"' && input_chars[idx] == '"' {
-            output_chars.push('"');
             idx += 1;
-        } else if input_chars[idx-1] == '\\' && input_chars[idx] == '"' {
-            output_chars.push('"');
-            idx += 1;
-        } else if input_chars[idx-1] == '\\' && input_chars[idx] == '\'' {
-            output_chars.push('\'');
-            idx += 1;
-        } else if input_chars[idx-1] == '\\' && input_chars[idx] == '\\' {
-            output_chars.push('\\');
-            idx += 1;
-        } else if input_chars[idx-1] == '\\' && input_chars[idx] == 'n' {
-            output_chars.push('\n');
-            idx += 1;
-        } else if input_chars[idx-1] == '\\' && input_chars[idx] == 'r' {
-            output_chars.push('\r');
-            idx += 1;
-        } else if input_chars[idx-1] == '\\' && input_chars[idx] == 't' {
-            output_chars.push('\t');
-            idx += 1;
-        } else {
+        }
+        if idx == input_chars.len() {
             output_chars.push(input_chars[idx-1]);
         }
 
-        idx += 1;
+        output_chars.iter().collect()
+    } else {
+        text.to_owned()
     }
-    if idx == input_chars.len() {
-        output_chars.push(input_chars[idx-1]);
-    }
-
-    output_chars.iter().collect()
 }
+
+
+
+#[test]
+fn parsing_numbers_test() {
+    let input_text = r##"0 0x1 1.0e+2 1000 0 0.1 0x11 1.0e+2"##;
+    let tokenresult = super::tokenizer::tokenize("test_input".to_owned(), 0, input_text);
+    assert!(tokenresult.is_ok());
+
+    let tokenresult = tokenresult.unwrap();
+    let mut log_msgs = Vec::<String>::new();
+    let mut parser = ParserState::new(&tokenresult.tokens, &tokenresult.filedata, &tokenresult.filenames, &mut log_msgs, true);
+    let context = ParseContext { element: "TEST".to_string(), fileid: 0, line: 0, inside_block: true };
+
+    // uint8: 0
+    let res = parser.get_integer_u8(&context);
+    assert!(res.is_ok());
+    let val = res.unwrap();
+    assert_eq!(val, (0, false));
+
+    // uint8: 0x1
+    let res = parser.get_integer_u8(&context);
+    assert!(res.is_ok());
+    let val = res.unwrap();
+    assert_eq!(val, (1, true));
+
+    // uint8: 1.0e+2
+    let res = parser.get_integer_u8(&context);
+    assert!(res.is_err());
+
+    // uint8: 257
+    let res = parser.get_integer_u8(&context);
+    assert!(res.is_err());
+
+    // float: 0
+    let res = parser.get_float(&context);
+    assert!(res.is_ok());
+    let val = res.unwrap();
+    assert_eq!(val, 0f32);
+
+    // float: 0.1
+    let res = parser.get_float(&context);
+    assert!(res.is_ok());
+    let val = res.unwrap();
+    assert_eq!(val, 0.1f32);
+
+    // float: 0x11
+    let res = parser.get_float(&context);
+    assert!(res.is_err());
+
+    // float: 1.0e+2
+    let res = parser.get_float(&context);
+    assert!(res.is_ok());
+    let val = res.unwrap();
+    assert_eq!(val, 100f32);
+}
+
+
+#[test]
+fn test_unescape_string() {
+    // no escape
+    let result = unescape_string(" ");
+    assert_eq!(result, " ");
+    // "" -> "
+    let result = unescape_string(r#""""#);
+    assert_eq!(result, r#"""#);
+    // \" -> "
+    let result = unescape_string(r#"\""#);
+    assert_eq!(result, r#"""#);
+    // \\ -> \
+    let result = unescape_string(r#"\\"#);
+    assert_eq!(result, r#"\"#);
+    // \n -> (newline)
+    let result = unescape_string(r#"\n"#);
+    assert_eq!(result, "\n");
+    // \r -> (carriage return)
+    let result = unescape_string(r#"\r"#);
+    assert_eq!(result, "\r");
+    // \t -> (tab)
+    let result = unescape_string(r#"\t"#);
+    assert_eq!(result, "\t");
+}
\ No newline at end of file
